/*
   Form render 自定义组件 :文件上传
*/
import React, { useCallback, useEffect, useState } from 'react';
import { UploadOutlined } from '@ant-design/icons';
import antdUI, { ajaxUpload } from './config.js';
const { Input, Row, Col, Button, Upload, message } = antdUI;
//解析文件格式
const parseFileList = (value, dataType) => {
  if (!value) {
    return [];
  }
  if (value.file && value.fileList) return value;

  let _filelist = [];
  if (dataType == 'object') {
    if (value.filename) {
      _filelist = [{ ...value, name: value.filename, uid: value.id || '-1' }];
    }
  } else {
    if (value && Array.isArray(value)) {
      _filelist = value?.map((v, index) => ({
        ...v,
        name: v?.filename,
        uid: v?.id || index,
      }));
    }
  }
  return _filelist;
};

const VUpload = (props) => {
  //console.log('widget props:', props);
  const { addons, schema, value, ...rest } = props;
  const [uploadProps, setUploadProps] = useState({});
  const [defaultFileList, setDefaultFileList] = useState(parseFileList(props.value, schema.type));

  useEffect(() => {
    let _temp = schema.props
      ? {
          ...schema.props,
        }
      : {};

    if (schema.type == 'object') {
      _temp.maxCount = 1;
    } else {
      _temp.maxCount = schema.props?.maxCount || 100;
      _temp.multiple = true;
    }
    setUploadProps(_temp);
  }, [schema]);
  //限制尺寸
  const beforeUpload = useCallback(
    (file) => {
      let isValid = true;
      const maxSize = schema.props?.maxsize;
      if (maxSize) {
        isValid = file.size < maxSize;
      }
      if (!isValid) {
        message.error(`文件不能超过${maxSize}(${maxSize / 1024}k)`);
      }
      return isValid || Upload.LIST_IGNORE;
    },
    [schema],
  );
  //请求API
  const customRequest = useCallback(
    (file) => {
      const data = new FormData();
      data.append('file', file.file);
      ajaxUpload(data)
        .then((res) => {
          file.onSuccess(res);
        })
        .catch((err) => {
          file.onError(err);
        });
    },
    [schema.type],
  );
  const handleChange = useCallback(
    (value) => {
      const { fileList, file } = value;
      const values = fileList.map((v) => {
        if (v.response?.data) {
          return { id: '', hash: v.response?.data, filename: v.name };
        }
      });
      //多个还是1个
      addons?.setValueByPath(addons.dataPath, schema.type == 'array' ? values : values[0]);
    },
    [schema.type],
  );
  return (
    <>
      <Upload
        beforeUpload={beforeUpload}
        customRequest={customRequest}
        defaultFileList={defaultFileList}
        onChange={handleChange}
        /* {...rest}  */
        {...uploadProps}
      >
        <Button icon={<UploadOutlined />}>上传</Button>
      </Upload>
    </>
  );
};
export default React.memo(VUpload);
