import React, { useEffect, useState, forwardRef, useImperativeHandle } from 'react';
import Form, { useForm } from 'form-render';
import VUpload from './VUpload.jsx';
import VArrayList from './VArrayList.jsx';

import antdUI, { enUS, zhCN } from './config';
/* import './index.scss'; */

//获取默认值
const getDataDefault = (type) => {
  const map = {
    string: '',
    array: [],
    boolean: false,
    number: 0,
    object: null,
    date: null,
    range: [],
  };
  return type ? map[type] : null;
};
const getWholeData = (schema, _data) => {
  if (!schema) return _data;
  let data = JSON.parse(JSON.stringify(_data));
  const keysData = Object.keys(data);
  const keysSchema = Object.keys(schema.properties);
  if (keysData.length < keysSchema.length) {
    keysSchema.forEach((key) => {
      if (!keysData.includes(key)) {
        data[key] = getDataDefault(schema.properties[key].type);
      }
    }); 
  }
  return data;
};

const VFormRender = forwardRef((props, ref) => {
  const form = useForm();
  const [locale] = useState(props.locale == 'en' ? enUS : zhCN);
  const [schema, setSchema] = useState({});
  //将子组件的方法 暴露给父组件
  useImperativeHandle(ref, () => ({
    submit: () => {
      return form.submit().then(
        ({ data, errors }, x) => {
          const wholeFormData = getWholeData(schema, data);
          if (errors.length > 0) {
            return Promise.reject({ data: wholeFormData, errors });
          }
          return Promise.resolve({ data: wholeFormData, errors });
        },
        ({ data, errors }) => {
          return Promise.reject({ data, errors });
        },
      );
    },
    getValues: form.getValues,
  }));
  useEffect(() => {
    form.setValues(props.schema?._formData || {});
    setSchema(props.schema);
  }, [props.schema]);
  const onFinish = (data, errors) => {
    if (props.onFinish && typeof onFinish == 'function') {
      const wholeFormData = getWholeData(schema, data);
      props.onFinish(wholeFormData, errors);
    }
  };
  return (
    <Form
      className="bas-formrender"
      widgets={{
        vupload: VUpload,
        varraylist: VArrayList,
        password: antdUI.Password,
      }}
      form={form}
      {...props}
      onFinish={onFinish}
      schema={schema}
      configProvider={{ locale }}
    />
  );
});
export default VFormRender;
